@@ -1,4 +1,4 @@
-use crate::args::{EDITOR_ENV, IGREP_EDITOR_ENV};
+use crate::args::{EDITOR_ENV, IGREP_EDITOR_ENV, VISUAL_ENV};
 use anyhow::{anyhow, Result};
 use clap::ArgEnum;
 use itertools::Itertools;
@@ -25,6 +25,12 @@ pub enum Editor {
     Emacsclient,
     Hx,
     Helix,
+    Subl,
+    SublimeText,
+    Micro,
+    Intellij,
+    Goland,
+    Pycharm,
 }
 
 impl Editor {
@@ -35,34 +41,29 @@ impl Editor {
                 .map(Editor::to_string)
                 .join(", ");
             anyhow!(e).context(format!(
-                "\"{}\" read from ${}, possible variants: [{}]",
-                env_value, env_name, possible_variants
+                "\"{env_value}\" read from ${env_name}, possible variants: [{possible_variants}]",
             ))
         };
 
-        if let Some(editor_cli) = editor_cli {
-            Ok(editor_cli)
-        } else if let Ok(value) = std::env::var(IGREP_EDITOR_ENV) {
-            let value = Editor::extract_editor_name(&value);
-            Editor::from_str(&value, false)
-                .map_err(|error| add_error_context(error, value, IGREP_EDITOR_ENV))
-        } else if let Ok(value) = std::env::var(EDITOR_ENV) {
-            let value = Editor::extract_editor_name(&value);
-            Editor::from_str(&value, false)
-                .map_err(|error| add_error_context(error, value, EDITOR_ENV))
-        } else {
-            Ok(Editor::default())
-        }
+        let read_from_env = |name| {
+            std::env::var(name).ok().map(|value| {
+                Editor::from_str(&Editor::extract_editor_name(&value), false)
+                    .map_err(|error| add_error_context(error, value, name))
+            })
+        };
+
+        editor_cli
+            .map(Ok)
+            .or_else(|| read_from_env(IGREP_EDITOR_ENV))
+            .or_else(|| read_from_env(VISUAL_ENV))
+            .or_else(|| read_from_env(EDITOR_ENV))
+            .unwrap_or(Ok(Editor::default()))
     }
 
-    pub fn spawn(self, file_name: &str, line_number: u64) -> Child {
+    pub fn spawn(self, file_name: &str, line_number: u64) -> io::Result<Child> {
         let mut command = EditorCommand::new(self, file_name, line_number);
-        command.spawn().unwrap_or_else(|_| {
-            panic!(
-                "Error: Failed to run editor with a command: \"{}\"",
-                command
-            );
-        })
+
+        command.spawn()
     }
 
     fn extract_editor_name(input: &str) -> String {
@@ -89,13 +90,19 @@ impl EditorCommand {
             Editor::CodeInsiders => "code-insiders".into(),
             Editor::Emacs => "emacs".into(),
             Editor::Emacsclient => "emacsclient".into(),
-            Editor::Hx | Editor::Helix => "hx".into(),
+            Editor::Hx => "hx".into(),
+            Editor::Helix => "helix".into(),
+            Editor::Subl | Editor::SublimeText => "subl".into(),
+            Editor::Micro => "micro".into(),
+            Editor::Intellij => "idea".into(),
+            Editor::Goland => "goland".into(),
+            Editor::Pycharm => "pycharm".into(),
         }
     }
 
     fn args(editor: Editor, file_name: &str, line_number: u64) -> Box<dyn Iterator<Item = String>> {
         match editor {
-            Editor::Vim | Editor::Neovim | Editor::Nvim | Editor::Nano => {
+            Editor::Vim | Editor::Neovim | Editor::Nvim | Editor::Nano | Editor::Micro => {
                 Box::new([format!("+{line_number}"), file_name.into()].into_iter())
             }
             Editor::Code | Editor::Vscode | Editor::CodeInsiders => {
@@ -107,6 +114,12 @@ impl EditorCommand {
             Editor::Hx | Editor::Helix => {
                 Box::new([format!("{file_name}:{line_number}")].into_iter())
             }
+            Editor::Subl | Editor::SublimeText => {
+                Box::new([format!("{file_name}:{line_number}")].into_iter())
+            }
+            Editor::Intellij | Editor::Goland | Editor::Pycharm => {
+                Box::new(["--line".into(), format!("{line_number}"), file_name.into()].into_iter())
+            }
         }
     }
 
@@ -138,22 +151,25 @@ mod tests {
         static ref SERIAL_TEST: std::sync::Mutex<()> = Default::default();
     }
 
-    #[test_case(Some("nano"), Some("vim"), Some("neovim") => matches Ok(Editor::Nano); "cli")]
-    #[test_case(None, Some("nano"), Some("neovim") => matches Ok(Editor::Nano); "igrep env")]
-    #[test_case(None, None, Some("nano") => matches Ok(Editor::Nano); "editor env")]
-    #[test_case(Some("unsupported-editor"), None, None => matches Err(_); "unsupported cli")]
-    #[test_case(None, Some("unsupported-editor"), None => matches Err(_); "unsupported igrep env")]
-    #[test_case(None, None, Some("unsupported-editor") => matches Err(_); "unsupported editor env")]
-    #[test_case(None, None, None => matches Ok(Editor::Vim); "default editor")]
-    #[test_case(None, Some("/usr/bin/nano"), None => matches Ok(Editor::Nano); "igrep env path")]
-    #[test_case(None, None, Some("/usr/bin/nano") => matches Ok(Editor::Nano); "editor env path")]
+    #[test_case(Some("nano"), Some("vim"), None, Some("neovim") => matches Ok(Editor::Nano); "cli")]
+    #[test_case(None, Some("nano"), None, Some("neovim") => matches Ok(Editor::Nano); "igrep env")]
+    #[test_case(None, None, Some("nano"), Some("helix") => matches Ok(Editor::Nano); "visual env")]
+    #[test_case(None, None, None, Some("nano") => matches Ok(Editor::Nano); "editor env")]
+    #[test_case(Some("unsupported-editor"), None, None, None => matches Err(_); "unsupported cli")]
+    #[test_case(None, Some("unsupported-editor"), None, None => matches Err(_); "unsupported igrep env")]
+    #[test_case(None, None, None, Some("unsupported-editor") => matches Err(_); "unsupported editor env")]
+    #[test_case(None, None, None, None => matches Ok(Editor::Vim); "default editor")]
+    #[test_case(None, Some("/usr/bin/nano"), None, None => matches Ok(Editor::Nano); "igrep env path")]
+    #[test_case(None, None, None, Some("/usr/bin/nano") => matches Ok(Editor::Nano); "editor env path")]
     fn editor_options_precedence(
         cli_option: Option<&str>,
         igrep_editor_env: Option<&str>,
+        visual_env: Option<&str>,
         editor_env: Option<&str>,
     ) -> Result<Editor> {
         let _guard = SERIAL_TEST.lock().unwrap();
         std::env::remove_var(IGREP_EDITOR_ENV);
+        std::env::remove_var(VISUAL_ENV);
         std::env::remove_var(EDITOR_ENV);
 
         let opt = if let Some(cli_option) = cli_option {
@@ -166,6 +182,10 @@ mod tests {
             std::env::set_var(IGREP_EDITOR_ENV, igrep_editor_env);
         }
 
+        if let Some(visual_env) = visual_env {
+            std::env::set_var(VISUAL_ENV, visual_env);
+        }
+
         if let Some(editor_env) = editor_env {
             std::env::set_var(EDITOR_ENV, editor_env);
         }
@@ -186,7 +206,13 @@ mod tests {
     #[test_case(Editor::Emacs => format!("emacs -nw +{LINE_NUMBER} {FILE_NAME}"); "emacs command")]
     #[test_case(Editor::Emacsclient => format!("emacsclient -nw +{LINE_NUMBER} {FILE_NAME}"); "emacsclient command")]
     #[test_case(Editor::Hx => format!("hx {FILE_NAME}:{LINE_NUMBER}"); "hx command")]
-    #[test_case(Editor::Helix => format!("hx {FILE_NAME}:{LINE_NUMBER}"); "helix command")]
+    #[test_case(Editor::Helix => format!("helix {FILE_NAME}:{LINE_NUMBER}"); "helix command")]
+    #[test_case(Editor::Subl => format!("subl {FILE_NAME}:{LINE_NUMBER}"); "subl command")]
+    #[test_case(Editor::SublimeText => format!("subl {FILE_NAME}:{LINE_NUMBER}"); "sublime text command")]
+    #[test_case(Editor::Micro => format!("micro +{LINE_NUMBER} {FILE_NAME}"); "micro command")]
+    #[test_case(Editor::Intellij => format!("idea --line {LINE_NUMBER} {FILE_NAME}"); "intellij command")]
+    #[test_case(Editor::Goland => format!("goland --line {LINE_NUMBER} {FILE_NAME}"); "goland command")]
+    #[test_case(Editor::Pycharm => format!("pycharm --line {LINE_NUMBER} {FILE_NAME}"); "pycharm command")]
     fn editor_command(editor: Editor) -> String {
         EditorCommand::new(editor, FILE_NAME, LINE_NUMBER).to_string()
     }
